@@ -1,18 +1,23 @@
 # -------------------------------------------------------------------------------
-# Copyright 2006-2021 UT-Battelle, LLC. See LICENSE for more information.
+# Copyright 2006-2022 UT-Battelle, LLC. See LICENSE for more information.
 # -------------------------------------------------------------------------------
 # local version
 import os
 import time
+from collections import namedtuple
 from math import ceil
 from .ipsExceptions import (InsufficientResourcesException,
                             BadResourceRequestException,
                             ResourceRequestMismatchException,
+                            GPUResourceRequestMismatchException,
                             ResourceRequestUnequalPartitioningException)
 from .ips_es_spec import eventManager
 from .resourceHelper import getResourceList
 from .node_structure import Node
 
+Allocation = namedtuple("Allocation",
+                        ["partial_node", "nodelist", "corelist", "ppn", "max_ppn", "cpp", "accurateNodes", "cores_allocated"])
+
 
 class ResourceManager:
     """
@@ -60,6 +65,7 @@ class ResourceManager:
         # other stuff
         self.max_ppn = 1   # the ppn for the whole submission (max ppn allowed by *software*)
         self.ppn = 1  # platform config ppn for the whole IPS
+        self.gpn = 0
         self.myTopic = None
         self.service_methods = ['get_allocation', 'release_allocation']
 
@@ -172,6 +178,11 @@ class ResourceManager:
                 self.sockets_per_node = 1
                 self.cores_per_socket = self.cores_per_node
 
+            # -------------------------------
+            # set gpp
+            # -------------------------------
+            self.gpn = int(self.CM.get_platform_parameter('GPUS_PER_NODE'))
+
         # -------------------------------
         # populate nodes
         # -------------------------------
@@ -256,7 +267,7 @@ class ResourceManager:
     # RM getAllocation
     # pylint: disable=inconsistent-return-statements
     def get_allocation(self, comp_id, nproc, task_id,
-                       whole_nodes, whole_socks, task_ppn=0, task_cpp=0):
+                       whole_nodes, whole_socks, task_ppn=0, task_cpp=0, task_gpp=0):
         """
         Traverse available nodes to return:
 
@@ -354,6 +365,11 @@ class ResourceManager:
                                                                   self.total_cores,
                                                                   self.max_ppn)
         else:
+            if not self.check_gpus(ppn, task_gpp):
+                raise GPUResourceRequestMismatchException(comp_id, task_id,
+                                                          ppn, task_gpp,
+                                                          self.gpn)
+
             try:
                 self.processes += nproc
                 cores_allocated = 0
@@ -443,10 +459,24 @@ class ResourceManager:
 
             if whole_nodes:
                 self.report_RM_status("allocation for task %d using whole nodes" % task_id)
-                return not whole_nodes, nodes, ppn, self.max_ppn, cpp, self.accurateNodes, cores_allocated
+                return Allocation(partial_node=not whole_nodes,
+                                  nodelist=nodes,
+                                  corelist=None,
+                                  ppn=ppn,
+                                  max_ppn=self.max_ppn,
+                                  cpp=cpp,
+                                  accurateNodes=self.accurateNodes,
+                                  cores_allocated=cores_allocated)
             else:
                 self.report_RM_status("allocation for task %d using partial nodes" % task_id)
-                return not whole_nodes, nodes, node_file_entries, ppn, self.max_ppn, self.accurateNodes, cores_allocated
+                return Allocation(partial_node=not whole_nodes,
+                                  nodelist=nodes,
+                                  corelist=node_file_entries,
+                                  ppn=ppn,
+                                  max_ppn=self.max_ppn,
+                                  cpp=None,
+                                  accurateNodes=self.accurateNodes,
+                                  cores_allocated=cores_allocated)
 
     def check_whole_node_cap(self, nproc, ppn):
         """
@@ -576,6 +606,9 @@ class ResourceManager:
         else:
             return False, "mismatch"
 
+    def check_gpus(self, ppn, task_gpp):
+        return ppn * task_gpp <= self.gpn
+
     # RM releaseAllocation
     def release_allocation(self, task_id, status):
         """
